# Created byMartin.cz
# Copyright (c) Martin Strohalm. All rights reserved.

# define tokens
API_KEY = None
USER_TOKEN = None

# define API urls
API_LEGO_URL = "https://rebrickable.com/api/v3/lego/"
API_USERS_URL = "https://rebrickable.com/api/v3/users/"

# define image urls
ELEMENT_IMG_URL = "https://m.rebrickable.com/media/parts/elements/{0}.jpg"
SET_IMG_URL = "https://m.rebrickable.com/media/sets/{0}.jpg"

# define minimum delay between requests in seconds
REQUEST_DELAY = 0.2
